=begin
(c) TIG 2010
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
___________________________________________________________________________

  exportgroups2objs.rb >>> ../Plugins/ folder
___________________________________________________________________________
Purpose:
It exports every 'visible' Group in the Model's entities as a separate OBJ 
file - 'visible' means it is not hidden and/or its layer is 'on'.
A nested Group within a Component is not exported unless that Component 
Instance is itself within another Group that is in the Model's entities.
A nested Group that is within another Group that itself is in the Model's 
entities will of course be included with the contents of the exported Group.
___________________________________________________________________________
Usage:
To run the tool type 'exportgroups2objs' in the Ruby Console.
If the Model SKP is new and has not yet been saved you will get an error.
If there are no Groups to Export as OBJs you will get an error.
All Groups are processed in turn - the OBJ file path in displayed in 
the Console.
The paired OBJ/MTL files are exported into the Model's folder.
The files' names are based on each Group's name with an incrementing suffix 
if there are more than one Group with that name - e.g. 
'Group_A_0001.obj'
'Group_A_0002.obj'
'Group_B_0001.obj'
If the exported Group has no name then it is named 'Group__0001.obj' etc...
On completion the total number of Groups exported as OBJs is reported.
___________________________________________________________________________
Donations:
by PayPal to info @ revitrev.org
___________________________________________________________________________
Version:
1.0 20101010 First release.
1.1 20101010 Disallowed characters in File_names from Group_names >> '_'.
___________________________________________________________________________
=end
require 'sketchup.rb'
###
def exportgroups2objs()
  model=Sketchup.active_model
  mpath=model.path
  if not mpath or mpath==""
    UI.beep
    puts "Save this new model before exporting its groups as OBJs..."
    return nil
  end
  mpath=File.dirname(mpath)
  ents=model.entities
  entsa=ents.to_a
  defns=model.definitions
  ss=model.selection
  ssa=ss.to_a
  groups=[]
  defns.each{|d|d.instances.each{|group|groups << group if d.group? and group.parent==model}}
  if not groups[0]
    UI.beep
    puts "No Groups to export..."
    return nil
  end
  names=[]
  groups.each{|group|
    ss.clear
    ss.add(entsa)
    ss.remove(group)
    tohide=ss.to_a
    ss.clear
    hidden=[]
    tohide.each{|e|
      if not e.hidden?
        e.hidden=true
        hidden << e
      end
    }
    name=("Group_"+group.name+"_0001").gsub(/[^A-Za-z0-9_]/,"_")
    while names.include?(name)
      name.next!
    end
    names << name
    obj=name+".obj"
    puts path=File.join(mpath, obj).tr("\\","/")
    model.export(path, false)
    hidden.each{|e|e.hidden=false}
  }
  UI.beep
  puts "Exported "+groups.length.to_s+" Groups as OBJs."
end
